* Prompt
*
* V0.01, pjw, 2001 Jan 27, gen. util for assempler routines
* V0.02, pjw, 2022 Feb 21, modified & improved for S*BASIC command

        section code

        xdef    sbu_prompt

        include dev8_keys_qdos_io

*
sbu_prompt
* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* Prints a prompt string and awaits a single key user-response
* The prompt string should be a short string (eg <Y/N/Q/A> etc)
* An list of option characters must also be provided in separate string pars.
* The first alternative should be a(n uppercase) printable character.
* The alternatives are compared directly with the input so if
* both upper and lower case options are wanted both will have to be
* specified.
* The function returns the option number 1 .. N, or -1 for an undefined ESC
*
* An example of the YN prompt should serve to illustrate:
*
* answ% = PROMPT('<Y/n>? ', 'Yy' & CHR$(10), 'Nn' & CHR$(27))
*
* A special "press any key" version is catered for: simply dont supply any
* options. The code of the key pressed is then returned.
* Another "special" is if only one option parameter is supplied:
* The key code is compared with the characters in the option string. If a
* match is found its code is returned, eg
*
* drv% = PROMPT("Enter drive no.: ", 12345678)
*
* If ESC is not specified in any option the routine returns with -1 if
* ESC is pressed.
*
* <BREAK> is active if the routine is called from S(uper)BASIC
*
* Parameter errors are "hard", ie they stop program execution.
*
* input :
*       a0   =  prompt channel id
*       a1   -> first string on RI stack
*       d4   =  number of options
*
* return: cc =  set
*       d4.l =  number of option chosen or keycode or -1
*       errors: i/o
*       d1-d3,d5-d6/a1-a4 smashed
*
        move.w  (a6,a1.l),d2    d2 = len prompt message

        moveq   #3,d0           prepare to find list of option strings
        add.w   d2,d0                (if any)
        bclr    #0,d0
        lea.l   (a1,d0.l),a2    a2 -> first option string length
        movea.l a2,a3           a3 =  keep copy for restart
        addq.l  #2,a1           a1 -> string bytes

        moveq   #iob.smul,d0    print prompt string
        trap    #4              make a6rel
        bsr.s   trap3i          write out prompt string
        beq.s   clearq

        rts

clearq                        ; clear keyboard queue
        moveq   #0,d3           timeout
cbloop
        moveq   #iob.fbyt,d0    getchar
        bsr.s   trap3
        beq.s   cbloop          continue until empty..

        moveq   #iow.ecur,d0    enable cursor
        bsr.s   trap3i

main
        movea.l a3,a4           a4 =  running pointer to options

        moveq   #iob.fbyt,d0    getchar
        bsr.s   trap3i          indefinite timeout
        bne.s   err_ret         could it be <break>?

        move.w  d4,d2           d2 = number of options
        beq.s   anykey          the "press any key" option

        subq.w  #1,d2           adjust for dbra

options
        move.w  (a6,a2.l),d5    number of chars in this option
        moveq   #3,d6
        add.w   d5,d6
        bclr    #0,d6           d6 = no chars to next option

        addq.l  #2,a2           chars start here
        move.b  (a6,a2.l),d3    keep main alternative
        bra.s   altend

alternative
        cmp.b   d1,d0
        beq.s   ok

        addq.l  #1,a2
altend
        move.b  (a6,a2.l),d0    subsequent alternative(s)
        dbra    d5,alternative

opt_end
        add.l   d6,a4           a4 -> next option
        movea.l a4,a2
        dbra    d2,options      go for next option

        cmpi.b  #27,d1          escape is always a valid option
        beq.s   esc

        movea.l a3,a2           re-start option list
        bsr.s   nvr             output "not a valid respons"
        bsr.s   pcol            backstep

        bra.s   main

pcol
        moveq   #iow.pcol,d0    back to previous column
        bra.s   trap3i

nvr
        cmpi.b  #' ',d1         printable?
        bcs.s   query

        cmpi.b  #191,d1
        bcs.s   char

query
        moveq   #'?',d1
char
        moveq   #iob.sbyt,d0    bput char
trap3i
        moveq   #-1,d3          timeout
trap3
        trap    #3
        tst.l   d0
        rts

esc
        moveq   #-1,d4          non-specified ESC pressed

escchr
        moveq   #'^',d1         ESC replacement character
        bsr.s   char
        bra.s   terminate

anykey
        moveq   #0,d4
        move.b  d1,d4
        cmpi.b  #27,d1          if it happens to be ESC, use ^
        beq.s   escchr

        bsr.s   nvr               otherwise print whatever
        bra.s   terminate

err_ret
        move.w  d0,d4           save error or return code
        bra.s   terminate

ok
        cmpi.w  #1,d4           if more than one option then
        bne.s   retopt           return option number
;                               else
        move.b  d1,d4            return key code
        bsr.s   char

        bra.s terminate

retopt
        move.b  d3,d1           output char of main option
        bsr.s   char            (assumed to be printable..)

        sub.w   d2,d4            return option number

terminate
        moveq   #10,d1          LF, ensure tidy end..
        bsr.s   char

        moveq   #iow.dcur,d0    switch off cursor
        bsr.s   trap3i

        moveq   #0,d0           all dandy
        rts

*
        end
